/* xrsi - (C) 2002 Hamish Rose 
 * Contact:             hgrose@users.sourceforge.net 
 * xrsi is released under the Artistic Licence.  Please read Licence.txt
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

/* handy data structure to store variables around in */
/* most elements can be easily found anyway, but this is a tad easier */
typedef struct all_data {
  Window win;
  Display* display;
  unsigned int display_width, display_height;  /* height and width of the X display.        */    
  unsigned int win_width, win_height;      /* height and width for the new window.      */
  GC gc;           /* GC (graphics context) used for drawing in our window   */
  XFontStruct* font_info;       /* Font structure, used for drawing text.    */
  int font_height;
  int screen_num;		/* number of screen to place the window on.  */
} all_data;

all_data data;

void setup_window(char *prog){
  unsigned long valuemask = 0;
  XGCValues values;		/* initial values for the GC.   */

  char* font_name = "*-helvetica-*-12-*"; /* font to use for drawing text.   */
  XTextProperty window_title_property;
  char* window_title = "xrsi";
  int rc;
  char *display_name = getenv("DISPLAY");  /* address of the X display.      */

  /* open connection with the X server. */
  data.display = XOpenDisplay(display_name);
  if (data.display == NULL) {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            prog, display_name);
    exit(1);
  }

  /* get the geometry of the default screen for our display. */
  data.screen_num = DefaultScreen(data.display);
  data.display_width = DisplayWidth(data.display, data.screen_num);
  data.display_height = DisplayHeight(data.display, data.screen_num);

  /* make the new window occupy 1/9 of the screen's size. */
  data.win_width = (data.display_width/3 );
  data.win_height = (data.display_height/3 );

  /* create a simple window, as a direct child of the screen's   */
  /* root window. Use the screen's white color as the background */
  /* color of the window. Place the new window's top-left corner */
  /* a third of the way into the screen from the top left corner of the display
   * */
  data.win = XCreateSimpleWindow(data.display, RootWindow(data.display,
	data.screen_num), data.win_width, data.win_height, data.win_width,
      data.win_height, 0, BlackPixel(data.display, data.screen_num),
      WhitePixel(data.display, data.screen_num));


  /* want to know if a button is pressed in the window */
  XSelectInput(data.display, data.win, ButtonPressMask);

  /* translate the given string into an X property. */
  rc = XStringListToTextProperty(&window_title, 1, &window_title_property);

  /* check the success of the translation. */
  if (rc == 0) {
    fprintf(stderr, "XStringListToTextProperty - out of memory\n");
    exit(1);
  }

  XSetWMName(data.display, data.win, &window_title_property);

  /* allocate a new GC for drawing in the window. */
  data.gc = XCreateGC(data.display, data.win, valuemask, &values);
  if (data.gc < 0) {
	fprintf(stderr, "XCreateGC: \n");
  }

  /* define the style of lines that will be drawn using this GC.  Linewidth is 2 */
  XSetLineAttributes(data.display, data.gc,
                     2, LineSolid, CapButt, JoinBevel);

  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(data.display, data.gc, FillSolid);
  XSync(data.display, False);

  /* try to load the given font. */
  data.font_info = XLoadQueryFont(data.display, font_name);
  if (!data.font_info) {
      fprintf(stderr, "XLoadQueryFont: failed loading font '%s'\n", font_name);
      exit(-1);
  }

  /* assign the given font to our GC. */
  XSetFont(data.display, data.gc, data.font_info->fid);
  /* find the height of the characters drawn using this font.        */
  data.font_height = data.font_info->ascent + data.font_info->descent;
}

void usage(char* name) {
  fprintf(stderr, "Usage: %s a b [c d]\n a: Time between micro pauses (seconds)\n b: Micro pause (seconds)\n c: Number of micro pauses before rest break\n d: Rest break (seconds) \n", name);
}

void draw_text(char *string1, char *string2) {
  int string_width, x, y;
  time_t timet=time(NULL);
  char text_string[100];

  /* find the width, in pixels, of the text */
  string_width = XTextWidth(data.font_info, string1, strlen(string1));
  x = (data.win_width - string_width) / 2;
  y = (data.win_height - 3* data.font_height) / 2;
  /* draw first string (Micro pause or Rest break) */
  XDrawString(data.display, data.win, data.gc, x, y, string1, strlen(string1));

  strftime(text_string, 100, "%l:%M %p", localtime(&timet));

  /* print the time on the window */
  string_width = XTextWidth(data.font_info, text_string, strlen(text_string));
  x = (data.win_width - string_width) / 2;
  y = (data.win_height - data.font_height) / 2;
  XDrawString(data.display, data.win, data.gc, x, y, text_string, strlen(text_string));

  /* draw second string (How long to break for) */
  string_width = XTextWidth(data.font_info, string2, strlen(string2));
  x = (data.win_width - string_width) / 2;
  y = (data.win_height + data.font_height) / 2;
  XDrawString(data.display, data.win, data.gc, x, y, string2, strlen(string2));

  XFlush(data.display);
}

void have_break(char *text, int length, int responsive) {
  char text_string[20];
  int i;
  XEvent event;
  /* draw the window on the display */
  XMapWindow(data.display, data.win);
  /* and put it in the middle */
  XMoveWindow(data.display, data.win, data.display_width/3, data.display_height/3);
  XFlush(data.display);

  /* grab the pointer.  Queue events until XUngrabPointer.  Don't force the
   * issue if it fails */
  XGrabPointer(data.display, RootWindow(data.display, data.screen_num), True, ButtonPressMask, GrabModeAsync,  GrabModeSync, None, None, CurrentTime);

  for (i=length; i>0; i--) {
    sprintf(text_string, "relax for %d seconds", i);
    draw_text(text, text_string);
    sleep(1);

    /* if a mouse button was pressed in our window */
    if (XCheckWindowEvent(data.display, data.win, ButtonPressMask, &event)) {
      if (responsive) i=0;	/* if its during a rest break, finish the break */
      else i++;			/* otherwise, add a second to the break time -
				   if users are pressing buttons they are not
				   taking a pause! */
      /* flush any remaining ButtonPress events from the queue */
      while (XCheckWindowEvent(data.display, data.win, ButtonPressMask,
	    &event)); }

    /* clear window */
    XClearWindow(data.display, data.win);
  }
  /* give back the pointer */
  XUngrabPointer(data.display, CurrentTime);
  /* get rid of the window */
  XUnmapWindow(data.display, data.win);
  XFlush(data.display);
}
    
int main(int argc, char* argv[]) {
  char text_string[20];
  int j;

  int WORK_TIME, PAUSE_TIME, N_MICRO_PAUSE, BREAK_TIME;

  /* check that we have 2 or 4 arguments */
  if (argc==3) {
    if (sscanf(argv[1],"%d", &WORK_TIME) != 1 || sscanf(argv[2],"%d", &PAUSE_TIME) !=1) {
      usage(argv[0]);
      return(1);
    }
    N_MICRO_PAUSE=1; 
    BREAK_TIME = -1; /* marker to indicate no rest breaks */
    /* check that all values make sense - greater than zero */
    if (WORK_TIME<=0 || PAUSE_TIME <=0 ) {
      fprintf(stderr, "All values must be postive!\n");
      usage(argv[0]);
      return(1);
    }
  } else if(argc==5)  {
    if (sscanf(argv[1],"%d", &WORK_TIME) != 1 || sscanf(argv[2],"%d", &PAUSE_TIME) !=1 || sscanf(argv[3],"%d", &N_MICRO_PAUSE) !=1 || sscanf(argv[4],"%d", &BREAK_TIME) !=1) {
      usage(argv[0]);
      return(1);
    }
    /* check that all values make sense - greater than zero */
    if (WORK_TIME<=0 || PAUSE_TIME <=0 || N_MICRO_PAUSE<=0 || BREAK_TIME<=0) {
      fprintf(stderr, "All values must be postive!\n");
      usage(argv[0]);
      return(1);
    }
  } else {
    usage(argv[0]);
    return(1);
  }

  setup_window(argv[0]);

  /* main loop */
  while (1) {
    for (j=0; j<N_MICRO_PAUSE; j++) {
      sleep(WORK_TIME);

      if (BREAK_TIME==-1)
	strcpy(text_string, "Micro pause");
      else
	sprintf(text_string, "Micro pause %d of %d", j+1, N_MICRO_PAUSE);

      have_break(text_string, PAUSE_TIME, 0);

      /* never leave this loop if there is no rest break */
      if (BREAK_TIME==-1) j=-1;

    }
    sleep(WORK_TIME);

    have_break("Rest Break", BREAK_TIME, 1);
  }

  /* close the connection to the X server, not that we ever reach here */
  XCloseDisplay(data.display);
  return 0;
}
